@@ -37,37 +37,44 @@
 setMethod("aggregate", signature(x="SpatRaster"),
 function(x, fact=2, fun="mean", ..., cores=1, filename="", overwrite=FALSE, wopt=list())  {
 
-	fun <- .makeTextFun(fun)
-	toc <- FALSE
-	if (inherits(fun, "character")) {
-		if (fun %in% c("sum", "mean", "min", "max", "median", "modal","prod", "which.min", "which.max",
-				"any", "all", "sd", "std", "sdpop")) {
-			fun[fun == "sdpop"] <- "std"
-			toc <- TRUE
+	# check if fact is an integer? round fact?
+
+	if (hasValues(x)) { 
+		fun <- .makeTextFun(fun)
+		toc <- FALSE
+		if (inherits(fun, "character")) {
+			if (fun %in% c("sum", "mean", "min", "max", "median", "modal","prod", "which.min", "which.max",
+					"any", "all", "none", "sd", "std", "sdpop", "table")) {
+				fun[fun == "sdpop"] <- "std"
+				toc <- TRUE
+			} else {
+				fun <- match.fun(fun)
+			}
 		} else {
 			fun <- match.fun(fun)
 		}
 	} else {
-		fun <- match.fun(fun)
+		toc = TRUE 
+		fun = "mean"
 	}
-	if (!hasValues(x)) { toc = TRUE }
+		
 	if (toc) {
 		#	fun="mean", expand=TRUE, na.rm=TRUE, filename=""
 		narm <- isTRUE(list(...)$na.rm)
 		opt <- spatOptions(filename, overwrite, wopt=wopt)
-		x@cpp <- x@cpp$aggregate(fact, fun, narm, opt)
+		x@pntr <- x@pntr$aggregate(fact, fun, narm, opt)
 		return (messages(x, "aggregate"))
 	} else {
 		out <- rast(x)
 		nl <- nlyr(out)
 		opt <- spatOptions()
-		out@cpp <- out@cpp$aggregate(fact, "sum", TRUE, opt)
+		out@pntr <- out@pntr$aggregate(fact, "sum", TRUE, opt)
 		out <- messages(out, "aggregate")
-		dims <- x@cpp$get_aggregate_dims(fact)
+		dims <- x@pntr$get_aggregate_dims(fact)
 
 		vtest <- values(x, dataframe=TRUE, row=1, nrows=dims[1], col=1, ncols=dims[2])
 		vtest <- as.list(vtest)
-		test <- sapply(vtest, fun)
+		test <- sapply(vtest, fun, ...)
 		dm <- dim(test)
 		do_transpose = FALSE
 		if (!is.null(dm)) {
@@ -118,7 +125,7 @@ function(x, fact=2, fun="mean", ..., cores=1, filename="", overwrite=FALSE, wopt
 		if (doPar) {
 			for (i in 1:b$n) {
 				v <- readValues(x, b$row[i], b$nrows[i], 1, nc)
-				v <- x@cpp$get_aggregates(v, b$nrows[i], dims)
+				v <- x@pntr$get_aggregates(v, b$nrows[i], dims)
 				v <- parallel::parSapply(cores, v, fun, ...)
 				if (length(v) != outnr[i] * mpl) {
 					error("aggregate", "this function does not return the correct number of values")
@@ -131,7 +138,7 @@ function(x, fact=2, fun="mean", ..., cores=1, filename="", overwrite=FALSE, wopt
 		} else {
 			for (i in 1:b$n) {
 				v <- readValues(x, b$row[i], b$nrows[i], 1, nc)
-				v <- x@cpp$get_aggregates(v, b$nrows[i], dims)
+				v <- x@pntr$get_aggregates(v, b$nrows[i], dims)
 				v <- sapply(v, fun, ...)
 				if (length(v) != outnr[i] * mpl) {
 					error("aggregate", "this function does not return the correct number of values")
@@ -212,7 +219,7 @@ setMethod("aggregate", signature(x="SpatVector"),
 		}
 		if (is.null(by)) {
 			x$aggregate_by_variable = 1;
-			x@cpp <- x@cpp$aggregate("aggregate_by_variable", dissolve)
+			x@pntr <- x@pntr$aggregate("aggregate_by_variable", dissolve)
 			x$aggregate_by_variable = NULL;
 		} else {
 			if (is.character(by)) {
@@ -245,7 +252,7 @@ setMethod("aggregate", signature(x="SpatVector"),
 				by <- names(x)[iby]
 			}
 
-			x@cpp <- x@cpp$aggregate(by, dissolve)
+			x@pntr <- x@pntr$aggregate(by, dissolve)
 			messages(x)
 
 			if (mvars) {
