@@ -4,6 +4,41 @@ unloadGDALdrivers <- function(x) {
 	.removeDriver(x)
 }
 
+proj_ok <- function() {
+	.proj_conf_test()
+}
+
+
+projNetwork <- function(enable, url="") {
+	if (missing(enable)) {
+		as.logical(.PROJ_network(-1, ""))
+	} else {
+		if (enable) {
+			out <- .PROJ_network(1, url)
+			if (out != "") {
+				message("PROJ network enabled. Using CDN URL: ", out)
+			} else {
+				message("PROJ network enabled")
+			}
+		} else {
+			.PROJ_network(0, "")
+			message("PROJ network disabled")
+		}
+		invisible(NULL)
+	}
+}
+
+
+projPaths <- function(paths, with_proj = TRUE) {
+	if (missing(paths)) {
+		.get_proj_search_paths()
+	} else {
+		paths <- as.character(paths)
+		.set_proj_search_paths(paths, with_proj)
+		invisible(.get_proj_search_paths())
+	}
+}
+
 
 fileBlocksize <- function(x) {
 	v <- x@pntr$getFileBlocksize()
@@ -179,8 +214,8 @@ ar_info <- function(x, what="describe", simplify=TRUE, filter=TRUE, array="") {
 setMethod("describe", signature(x="character"),
 	function(x, sds=FALSE, meta=FALSE, parse=FALSE, options="", print=FALSE, open_opt="") {
 
-		#x <- .fullFilename(x[1], FALSE)
-		x <- x[1]
+		x <- .fullFilename(x[1], FALSE)
+		#x <- x[1]
 		if (meta) {
 			if (sds) {
 				return(.meta_sds(x, parse))
