@@ -24,14 +24,24 @@
 
 .gdinit <- function() {
 	path = ""
-	sf <- system.file("", package="terra")
-	if (file.exists(file.path(sf, "proj/nad.lst"))) {
-		path <- system.file("proj", package="terra")
+	proj_path <- system.file("proj", package="terra")
+	gdal_path <- system.file("gdal", package="terra")
+	
+	has_proj <- dir.exists(proj_path) && (file.exists(file.path(proj_path, "proj.db")) || file.exists(file.path(proj_path, "nad.lst")))
+	has_gdal <- dir.exists(gdal_path)
+	
+	if (has_proj) {
+		projPaths(proj_path, TRUE)
 	}
-	.gdalinit(path, file.path(sf, "gdal"))
+	
+	.gdalinit(path, ifelse(has_gdal, gdal_path, ""))
+	
 	if (libVersion("gdal") == "3.6.0") {
 		message("Using GDAL version 3.6.0 which was retracted because it cannot write large GPKG files")
 	}
+	if (!proj_ok()) {
+		message("There appears to be a problem with the PROJ installation")
+	}
 }
 
 loadModule("spat", TRUE)
