@@ -12,6 +12,9 @@
 //#define GEOS_USE_ONLY_R_API
 #include <geos_c.h>
 
+#if defined(HAVE_TBB) && !defined(__APPLE__)
+#define USE_TBB
+#endif
 
 
 #if GDAL_VERSION_MAJOR >= 3
@@ -39,10 +42,29 @@
 #endif
 
 
+// [[Rcpp::export(name = ".have_TBB")]]
+bool have_TBB() {
+	#ifdef HAVE_TBB
+		return true;
+	#else 
+		return false;
+	#endif 
+}
+
+
+// [[Rcpp::export(name = ".proj_conf_test")]]
+bool proj_conf_test() {
+    PJ *crs = proj_create(nullptr, "EPSG:4326");
+    if (!crs) {
+        return false;
+    }
+    proj_destroy(crs);
+    return true;
+}
+
+
 //from sf
-
 #ifdef projh
-
 // [[Rcpp::export]]
 std::string proj_version() {
 	std::stringstream buffer;
@@ -211,10 +233,15 @@ std::string gdal_getconfig(std::string option) {
 
 // [[Rcpp::export(name = ".gdalinfo")]]
 std::string ginfo(std::string filename, std::vector<std::string> options, std::vector<std::string> oo) {
-	std::string out = gdalinfo(filename, options, oo);
-	return out;
+	return gdalinfo(filename, options, oo);
 }
 
+// [[Rcpp::export(name = ".gdalmdinfo")]]
+std::string gmdinfo(std::string filename, std::vector<std::string> options) {
+	return gdalMDinfo(filename, options);
+}
+
+
 // [[Rcpp::export(name = ".sdinfo")]]
 std::vector<std::vector<std::string>> sd_info(std::string filename) {
 	std::vector<std::vector<std::string>> sd = sdinfo(filename);
@@ -271,7 +298,7 @@ std::vector<std::vector<std::string>> gdal_drivers() {
     GDALDriver *poDriver;
     char **papszMetadata;
 	for (size_t i=0; i<n; i++) {
-	    poDriver = GetGDALDriverManager()->GetDriver(i);
+	    poDriver = GetGDALDriverManager()->GetDriver((int)i);
 		const char* ss = poDriver->GetDescription();
 		if (ss != NULL ) s[0][i] = ss;
 		ss = poDriver->GetMetadataItem( GDAL_DMD_LONGNAME );
@@ -407,14 +434,14 @@ void gdal_init(std::string projpath, std::string datapath) {
 	//GDAL_NETCDF_IGNORE_XY_AXIS_NAME_CHECKS
 
 	//GDALregistred = true;
-#if GDAL_VERSION_MAJOR >= 3
- #ifdef PROJ_6
 	if (!projpath.empty()) {
-		const char *cp = projpath.c_str();
-		proj_context_set_search_paths(PJ_DEFAULT_CTX, 1, &cp);
-	}
- #endif
+#if GDAL_VERSION_NUM >= 3000000
+		std::vector<char *> cpaths(2);
+		cpaths[0] = (char *)projpath.c_str();
+		cpaths[1] = NULL;
+		OSRSetPROJSearchPaths(cpaths.data());
 #endif
+	}
 #ifdef PROJ_71
 	#ifndef __EMSCRIPTEN__
 		proj_context_set_enable_network(PJ_DEFAULT_CTX, 1);
@@ -446,7 +473,7 @@ std::vector<double> percRank(std::vector<double> x, std::vector<double> y, doubl
 					break;
 				}
 			}
-			double z = (b + 0.5 * t) / nx;
+			double z = ((double)b + 0.5 * (double)t) / (double) nx;
 			if (tail == 1) { // both
 				if (z > 0.5) {
 					z = 2 * (1 - z);
@@ -482,7 +509,7 @@ void clearVSIcache(bool vsi) {
 // [[Rcpp::export(name = ".setGDALCacheSizeMB")]]
 void setGDALCacheSizeMB(double x, bool vsi) {
 	if (vsi) {
-		int64_t v = x * 1024 * 1024;
+		int64_t v = int64_t(x * 1024 * 1024);
 		CPLSetConfigOption("CPL_VSIL_CURL_CACHE_SIZE", std::to_string(v).c_str());
 	} else {
 		GDALSetCacheMax64(static_cast<int64_t>(x) * 1024 * 1024);
@@ -540,40 +567,78 @@ std::vector<std::string> get_proj_search_paths() {
 
 
 // [[Rcpp::export(name = ".set_proj_search_paths")]]
-bool set_proj_search_paths(std::vector<std::string> paths) {
+bool set_proj_search_paths(std::vector<std::string> paths, bool with_proj = false) {
 	if (paths.empty()) {
 		return false;
 	}
+	if (with_proj) {
+		// Set for PROJ library
+		if (paths.size() == 1) {
+			const char *cp = paths[0].c_str();
+			proj_context_set_search_paths(PJ_DEFAULT_CTX, 1, &cp);
+		}
+		return true;
+	} else {
+		// Set for GDAL
 #if GDAL_VERSION_NUM >= 3000000
-	std::vector <char *> cpaths(paths.size()+1);
-	for (size_t i = 0; i < paths.size(); i++) {
-		cpaths[i] = (char *) (paths[i].c_str());
-	}
-	cpaths[cpaths.size()] = NULL;
-	OSRSetPROJSearchPaths(cpaths.data());
-	return true;
+		std::vector <char *> cpaths(paths.size()+1);
+		for (size_t i = 0; i < paths.size(); i++) {
+			cpaths[i] = (char *) (paths[i].c_str());
+		}
+		cpaths[cpaths.size()-1] = NULL;
+		OSRSetPROJSearchPaths(cpaths.data());
+		return true;
 #else
-	return false;
+		return false;
 #endif
+	}
 }
 
+
 // [[Rcpp::export(name = ".PROJ_network")]]
-std::string PROJ_network(bool enable, std::string url) {
+std::string PROJ_network(int enable, std::string url) {
 	std::string s = "";
 #ifdef PROJ_71
-	if (enable) {
+	if (enable == -1) { // get current status
+		return std::to_string(proj_context_is_network_enabled(PJ_DEFAULT_CTX));
+	} else if (enable == 1) {
 		proj_context_set_enable_network(PJ_DEFAULT_CTX, 1);
+#if GDAL_VERSION_NUM >= 3040000
+		OSRSetPROJEnableNetwork(1);
+#endif
 		if (url.size() > 5) {
 			proj_context_set_url_endpoint(PJ_DEFAULT_CTX, url.c_str());
 		}
 		s = proj_context_get_url_endpoint(PJ_DEFAULT_CTX);
-	} else { // disable:
+	} else if (enable == 0) { // disable:
 		proj_context_set_enable_network(PJ_DEFAULT_CTX, 0);
+#if GDAL_VERSION_NUM >= 3040000
+		OSRSetPROJEnableNetwork(0);
+#endif
 	}
 #endif
 	return s;
 }
 
+
+
+// [[Rcpp::export(name = ".removeDriver")]]
+void removeDriver(std::vector<std::string> d) {
+	if ((d.size() == 0) || ((d.size() == 1) && (d[0] == ""))) {
+		GDALAllRegister();	
+	} else {
+		for (size_t i=0; i<d.size(); i++) {
+			GDALDriverH hDrv = GDALGetDriverByName(d[i].c_str());
+			if (hDrv == NULL) {
+				Rcpp::warning(d[i] + " is not a known driver\n");
+			} else {
+				GDALDeregisterDriver(hDrv);
+			}
+		}
+	}
+}
+
+
 // [[Rcpp::export(name = ".pearson")]]
 double pearson_cor(std::vector<double> x, std::vector<double> y, bool narm) {
  
@@ -590,8 +655,8 @@ double pearson_cor(std::vector<double> x, std::vector<double> y, bool narm) {
 		}
 	}
 	size_t n = x.size();
-	double xbar = accumulate(x.begin(), x.end(), 0.0) / n;
-	double ybar = accumulate(y.begin(), y.end(), 0.0) / n;
+	double xbar = accumulate(x.begin(), x.end(), 0.0) / (double)n;
+	double ybar = accumulate(y.begin(), y.end(), 0.0) / (double)n;
 	double numer = 0;
 	for (size_t i=0; i<n; i++) {
 		numer += (x[i]-xbar) * (y[i]-ybar);
@@ -654,7 +719,7 @@ Rcpp::IntegerMatrix uniqueSymmetricRows(std::vector<size_t> x, std::vector<size_
 	size_t n = x.size();
 	for (size_t i=0; i<n; i++) {
 		if (x[i] > y[i]) {
-			double tmp = x[i];
+			size_t tmp = x[i];
 			x[i] = y[i];
 			y[i] = tmp;
 		}
@@ -703,3 +768,108 @@ double stattest2(std::vector<double> x, std::string fun, bool narm) {
 
 */
 
+
+Rcpp::List get_output(std::vector<std::string> names, std::vector<long> sizes) {
+	Rcpp::List L = Rcpp::List::create(Rcpp::Named("name") = names, Rcpp::Named("size") = sizes);
+	return(L);
+}
+
+
+#if GDAL_VERSION_MAJOR >= 3 && GDAL_VERSION_MINOR >= 4
+
+// [[Rcpp::export(name = ".arnames")]]
+std::vector<std::string> arnames(std::string filename, bool filter) {
+// FROM GDAL 3.11 (while not widely available).
+// * Author:   Even Rouault <even.rouault at spatialys.com>
+// * Copyright (c) 2019, Even Rouault <even.rouault at spatialys.com>
+
+    std::vector<std::string> ret;
+    auto poDataset = std::unique_ptr<GDALDataset>(GDALDataset::Open(filename.c_str(), GDAL_OF_MULTIDIM_RASTER ));
+
+    if (!poDataset) {
+		ret.push_back("not a good dataset");
+        return ret;
+    }
+
+	std::shared_ptr<GDALGroup> poRootGroup = poDataset->GetRootGroup();
+    if (!poRootGroup) {
+		ret.push_back("no root group");
+        return ret;
+    }
+
+    std::list<std::shared_ptr<GDALGroup>> stackGroups;
+    stackGroups.push_back(nullptr);  // nullptr means this
+    while (!stackGroups.empty()) {
+        std::shared_ptr<GDALGroup> groupPtr = std::move(stackGroups.front());
+        stackGroups.erase(stackGroups.begin());
+        const GDALGroup *poCurGroup = groupPtr ? groupPtr.get() : poRootGroup.get();
+        for (const std::string &arrayName :  poCurGroup->GetMDArrayNames(nullptr)) {
+            std::string osFullName = poCurGroup->GetFullName();
+            if (!osFullName.empty() && osFullName.back() != '/') 
+                osFullName += '/';
+            osFullName += arrayName;
+            ret.push_back(std::move(osFullName));
+        }
+        auto insertionPoint = stackGroups.begin();
+        for (const auto &osSubGroup : poCurGroup->GetGroupNames(nullptr)) {
+            auto poSubGroup = poCurGroup->OpenGroup(osSubGroup);
+            if (poSubGroup)
+                stackGroups.insert(insertionPoint, std::move(poSubGroup));
+        }
+    }
+	if (filter) {
+		ret = ncdf_filternames(ret);
+	}
+    return ret;
+}
+
+
+// [[Rcpp::export(name = ".dimfo")]]
+Rcpp::List dimfo(std::string filename, std::string array_name) {
+
+	std::vector<std::string> names;
+	std::vector<long> sizes;
+
+    auto poDataset = std::unique_ptr<GDALDataset>(GDALDataset::Open(filename.c_str(), GDAL_OF_MULTIDIM_RASTER ));
+    if (!poDataset) {
+		names.push_back("cannot open as md: " + filename);
+		sizes.push_back(-99);
+		return get_output(names, sizes);
+    }
+
+	std::shared_ptr<GDALGroup> poRootGroup = poDataset->GetRootGroup();
+    if (!poRootGroup) {
+		names.push_back("no root group: " + filename);
+		sizes.push_back(-99);
+		return get_output(names, sizes);
+    }
+
+	auto poVar = poRootGroup->OpenMDArray(array_name.c_str());
+	if (!poVar)   {
+		names.push_back("cannot open: " + array_name);
+		sizes.push_back(-99);
+		return get_output(names, sizes);
+	}
+
+	if (names.size() == 0) {
+		for ( const auto &poDim: poVar->GetDimensions() ) {
+			names.push_back(static_cast<std::string>(poDim->GetName()));
+			sizes.push_back(static_cast<long>(poDim->GetSize()));
+		}
+	}
+	return get_output(names, sizes);
+}
+
+#else
+
+Rcpp::List dimfo(std::string filename, std::string array_name) {
+	return get_output({"not supported with GDAL < 3.4"}, {-99});
+}
+
+std::vector<std::string> arnames(std::string filename, bool filter) {
+	std::vector<std::string> out(1, "not supported with GDAL < 3.4");
+	return out;
+}
+
+#endif
+
