@@ -621,6 +621,8 @@ class SpatRaster {
 		void collapse();
 		
 		SpatRaster rectify(std::string method, SpatRaster aoi, unsigned useaoi, bool snap, SpatOptions &opt);
+
+		std::vector<double> centroid(bool weights, SpatOptions &opt);
 		
         std::vector<double> adjacent(std::vector<double> cells, std::string directions, bool include);
         std::vector<double> adjacentMat(std::vector<double> cells, std::vector<bool> mat, std::vector<size_t> dim, bool include);
@@ -662,7 +664,7 @@ class SpatRaster {
 		std::vector<double> bilinearCells(const std::vector<double> &x, const std::vector<double> &y);
 		void fourCellsFromXY(std::vector<double> &out, const std::vector<double> &x, const std::vector<double> &y);
 
-		SpatRaster buffer(double d, double background, SpatOptions &opt);
+		SpatRaster buffer(double d, double background, bool include, SpatOptions &opt);
 		SpatRaster clamp(std::vector<double> low, std::vector<double> high, bool usevalue, SpatOptions &opt);
 		SpatRaster clamp_raster(SpatRaster &x, SpatRaster &y, std::vector<double> low, std::vector<double> high, bool usevalue, SpatOptions &opt);
 		SpatRaster clamp_ts(bool min, bool max, SpatOptions &opt);
@@ -702,7 +704,7 @@ class SpatRaster {
 		SpatRaster patches(size_t directions, SpatOptions &opt);
 
 
-		SpatRaster edges(bool classes, std::string type, unsigned directions, double falseval, SpatOptions &opt);
+		SpatRaster edges(bool classes, bool internal, std::string type, unsigned directions, double falseval, SpatOptions &opt);
 		SpatRaster extend(SpatExtent e, std::string snap, double fill, SpatOptions &opt);
 		std::vector<std::vector<std::vector<double>>> extractVector(SpatVector v, bool touches, bool small, std::string method, bool cells, bool xy, bool weights, bool exact, SpatOptions &opt);
 		std::vector<double> extractVectorFlat(SpatVector v, std::vector<std::string> funs, bool narm, bool touches, bool small, std::string method, bool cells, bool xy, bool weights, bool exact, SpatOptions &opt);
@@ -754,20 +756,20 @@ class SpatRaster {
 		std::vector<std::string> getDataType(bool unique, bool memtype);
 		std::vector<std::string> dataType();
 
+		SpatRaster is_wrapper(std::string method, bool falseNA, SpatOptions &opt);
+
 		SpatRaster isnot(bool falseNA, SpatOptions &opt);
 		SpatRaster isnan(bool falseNA, SpatOptions &opt);
 		SpatRaster isnotnan(bool falseNA, SpatOptions &opt);
-		SpatRaster countnan(long n, SpatOptions &opt);
-
 		SpatRaster isfinite(bool falseNA, SpatOptions &opt);
 		SpatRaster isinfinite(bool falseNA, SpatOptions &opt);
 		SpatRaster is_true(bool falseNA, SpatOptions &opt);
 		SpatRaster is_false(bool falseNA, SpatOptions &opt);
-		SpatRaster not_na(bool falseNA, SpatOptions &opt);
-
 		SpatRaster allnan(bool falseNA, SpatOptions &opt);
 		SpatRaster anynan(bool falseNA, SpatOptions &opt);
 		SpatRaster nonan(bool falseNA, SpatOptions &opt);
+
+		SpatRaster countnan(long n, SpatOptions &opt);
 		SpatRaster which(SpatOptions &opt);
 
 		std::vector<std::vector<double>> layerCor(std::string fun, std::string use, bool asSample, SpatOptions &opt);
@@ -802,7 +804,7 @@ class SpatRaster {
 
         std::vector<double> polygon_cells(SpatGeom& g);
 		SpatRaster quantile(std::vector<double> probs, bool narm, SpatOptions &opt);
-		SpatRaster stretch(std::vector<double> minv, std::vector<double> maxv, std::vector<double> minq, std::vector<double> maxq, std::vector<double> smin, std::vector<double> smax, SpatOptions &opt);
+		SpatRaster stretch(std::vector<double> minv, std::vector<double> maxv, std::vector<double> minq, std::vector<double> maxq, std::vector<double> smin, std::vector<double> smax, bool bylayer, double maxcell, SpatOptions &opt);
 		SpatRaster reverse(SpatOptions &opt);
 
 		SpatRaster range(std::vector<double> add, bool narm, SpatOptions &opt);
